      Program NGA2SMC

* Reformat into a smc file an ascii file made by NGA.

* Sample control file:
* first line below starts the control, with line 1, col 1 = "N" in "Nominal":
*Control file for NGA2SMC      ! first line
*Name of summary file:
* nga2smc.sum
*Xfactr
* 981.0
*Nlines to skip before line with n_ts, dt
* 3
*Input file name ("stop" in any column to quit):
* \smsim\ts0.col
* \smsim\ts1.col
*STOP

* Dates: 05/13/04 - Written by D. Boore, patterned after asc2smc
*        12/19/04 - output file made by adding "smc" to end of input file name.
*        01/06/05 - NGA files have put the n_ts and dt on line 4 before the
*                   text, thus allowing free format read.
*        10/06/06 - Read number of lines before n_ts, dt from control file
*                   (because NGA files rotated to FN, FP have a few extra lines)
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

      character precision*8
      
      real time(100000), ts(100000)
      character f_ctl*40, f_sum*40, f_asc*40, f_smc*40, f_asc_4*4
      character char_head(11)*80, comments(100)*80, hdr_cmmnts(100)*79
      character headers_query*10
      integer int_head(48)
      real real_head(50)

      logical f_ctl_exist

      do i = 2, 11
        char_head(i) = '*'
      end do

      int_null = -32768
      do i = 1, 48
        int_head(i) = int_null
      end do
*      int_head(13) = 0
*      int_head(14) = 0

      real_null = 1.7e+38
      do i = 1, 50
        real_head(i) = real_null
      end do

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = nga2smc.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'nga2smc.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

      call skip(nu_ctl,2)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum, file=f_sum(1:nc_f_sum), status='unknown')
      write(nu_sum,'(a)') '  Summary program for program NGA2SMC'      

      call skip(nu_ctl,1)
      read(nu_ctl,*) xfactr

      call skip(nu_ctl,1)
      read(nu_ctl,*) nskip

      call skip(nu_ctl,1)

1234  continue

      f_asc = ' '
      read(nu_ctl,'(a)') f_asc
      call trim_c(f_asc,nc_f_asc)
      f_asc_4 = ' '
      f_asc_4 = f_asc(1:4)
      call upstr(f_asc_4)
      if(f_asc_4 .eq. 'STOP') go to 4321
      write(nu_sum,'(2a)') '  Processing file: ', f_asc(1:nc_f_asc)

      call get_lun(nu_asc)
      open(unit=nu_asc, file=f_asc(1:nc_f_asc), status='unknown')

      call skip(nu_asc, nskip)

      read(nu_asc,*) n_ts, dt
*      read(nu_asc,'(t6,i6, t17,f7.5)') n_ts, dt

      read(nu_asc,'(5e15.7)') (ts(i),i=1,n_ts)


* Scale the time series:
      do i = 1, n_ts
        ts(i) = xfactr * ts(i)
      end do

      sps = 1.0/dt

      write(nu_sum,'(a,1x,i6,1x,f7.2)') '    n_ts, sps = ', n_ts, sps

      int_head(17) = n_ts
      
      real_head(2) = sps

      call skipcmnt(nu_asc, hdr_cmmnts, n_hdr_cmmnts)

      ncomments = 0
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      do i = 1, n_hdr_cmmnts
        ncomments = ncomments + 1
        comments(ncomments) = '|'//hdr_cmmnts(i)
      end do
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      comments(ncomments) = '| File '//f_asc(1:nc_f_asc)//
     : ' reformatted by program NGA2SMC'
      ncomments = ncomments + 1
      comments(ncomments) = '| Data values scaled by the factor '
      call trim_c(comments(ncomments), nc_c)
      write(comments(ncomments)(nc_c+1:nc_c+11),'(0p,1x,e10.3)')
     :  xfactr
      ncomments = ncomments + 1
      comments(ncomments) = '|'

      int_head(16) = ncomments

      f_smc = ' '
      f_smc = f_asc(1:nc_f_asc)//'.smc'
      call trim_c(f_smc,nc_f_smc)

      write(*,'(a)') ' Writing file '//f_smc(1:nc_f_smc)

      precision = ' '
      precision = 'STANDARD'
      int_head(47) = int_head(1)

      call smcwrite(f_smc, precision,
     :               ts, char_head, 0, 
     :              int_head, real_head, comments)

      go to 1234

4321  continue

      close(nu_ctl)
      close(nu_sum)

      stop
      end

      include '\forprogs\skip.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\skipcmnt.for'
