      Program SAC2SMC

* Reformat into a smc file a binary file written in SAC format

* The smc file name is constructed by appending ".smc" to the input file name.

* Sample control file:
*!Control file for SAC2SMC      ! first line
*!Name of summary file:
* sac2smc.sum
*!Xfactr
* 1.0e-7   ! convert nanometers to cm (assume SAC data is in nanometers)
*!Input file name ("stop" in any column to quit):
* STIL.HNE.NM
* 14433456.NP.5271.HLN.sac
* 14433456.NP.5271.HLE.sac
* 14433456.NP.5271.HLZ.sac
*STOP
 
* Dates: 04/23/09 - Written by D. Boore, relying heavily on sacread.f90 by 
*                   Chris Stephens.  I also used sacread2.f by
*                   Karen Assatourians.  Chris's program is written in 
*                   Fortran 90 and the byte swapping is clearer in his code.
!        11/03/15 - If Xfactr < 0, attempt to use scale factor from header, 
!                   if available, following a suggestion
!                   by Morteza Dehghani.  According to him, the conversion is 
!                   100.0/head1(1,4) (the "100" is to convert from m/s/s to cm/s/s).
!                   If the scale factor is unassigned (head1(1,4) = -12345.0), 
!                   then set Xfactr = 1.0 
 
      USE byte_swaps

      character precision*8
      
      REAL(4) head1(14,5), dt
      REAL(4), DIMENSION(:), allocatable      :: ts
      integer(4) head2(8,5), julian_day, n_ts
      CHARACTER(LEN=8), dimension(8,3) :: head3

      character f_ctl*80, f_sum*80, f_sac*120, f_smc*120, f_sac_4*4
      character buf*80
      character 
     : char_head(11)*80, comments(100)*80 
      character cmnts2skip(20)*80
      character headers_query*10
      integer int_head(48) 
      real real_head(50)

      LOGICAL :: byte_swap = .false.

      logical f_ctl_exist

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = sac2smc.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'sac2smc.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      
      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum, file=f_sum(1:nc_f_sum), status='unknown')
      write(nu_sum,'(a)') '  Summary program for program sac2SMC'      

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)
      read(nu_ctl,*) xfactr_in

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)

1234  continue

      print *

        f_sac = ' '
        read(nu_ctl,'(a)') f_sac
        call trim_c(f_sac,nc_f_sac)
        f_sac_4 = ' '
        f_sac_4 = f_sac(1:4)
        call upstr(f_sac_4)
        if(f_sac_4 .eq. 'STOP') go to 4321
        write(nu_sum,'(2a)') '  Processing file: ', f_sac(1:nc_f_sac)

        call get_lun(nu_sac)
        open(unit=nu_sac,file=f_sac(1:nc_f_sac),status='old',
     :       form='binary')

	read(nu_sac)((head1(i,j),j=1,5),i=1,14)
 	read(nu_sac)((head2(i,j),j=1,5),i=1,8)

! test for byte swap

        if ((head2(1,2) .lt. 1 .or. head2(1,2) .gt. 366)               ! Chris's criterion, based on Julian day
     :                      .and.
     :     (head1(1,1) .lt. 0.0001 .or. head1(1,1) .gt. 1000.0) ) then ! Karen's criterion, based on sampling interval  
 
          byte_swap = .true.
          PRINT *, ' Do byte swapping'
          write(nu_sum,'(a)') ' Do byte swapping'
        endif

        IF (byte_swap) then

          do i = 1, 14
            do j = 1, 5
              head1(i,j) = r4swap(head1(i,j))
            end do
          end do

          do i = 1, 8
            do j = 1, 5
              head2(i,j) = i4swap(head2(i,j))
            end do
          end do

        endif

 	read(nu_sac) ((head3(i,j),j=1,3),i=1,8) ! may want to check for certain characters; see sacread2.f
 	
	do i = 1, 8
	  do j = 1, 3
	    call putspace(head3(i,j))
 	  end do
	end do
	    
        julian_day = head2(1,2) 
        
        dt = head1(1,1)
        
 	n_ts = head2(2,5)
 
        sps = 1.0/dt
        write(nu_sum,'(a,1x,i6,1x,f7.2)') '    n_ts, sps = ', n_ts, sps

	ALLOCATE( ts(n_ts) )

        READ(nu_sac) ts

        if (byte_swap) then

          do i = 1, n_ts
            ts(i) = r4swap(ts(i))
          end do

        end if

* Scale the time series:

        if (xfactr_in < 0) then
          if (head1(1,4) /= -12345.00) then
            xfactr = 100.0/head1(1,4)
          else
            xfactr = 1.0
          end if
        else
          xfactr = xfactr_in
        end if
        
        write(nu_sum, '(1x,a, 3(1x,es12.5))') 
     :      ' xfactr_in, head1(1,4), xfactr = ',
     :        xfactr_in, head1(1,4), xfactr
            
          
        do i = 1, n_ts
          ts(i) = xfactr * ts(i)
        end do

        do i = 2, 11
          char_head(i) = '*'
        end do
     
        int_null = -32768
        do i = 1, 48
            int_head(i) = int_null
        end do
        
!        int_sac_null = -12345
!        do i = 1, 8
!          do j = 1, 5
!            if(head2(i,j) .eq. int_sac_null) then
!              head2(i,j) = int_null
!            end if
!          end do
!        end do
        
        int_head(2) = head2(1,1)  !year
        int_head(3) = head2(1,2)  !julian day
        int_head(4) = head2(1,3)  !hour
        int_head(5) = head2(1,4)  !minute
        int_head(6) = head2(1,5)  !second
        
        if (head1(12,4) .eq. 0.0 .or. head1(12,4) .eq. 180.0) then ! a vertical component
          if (head2(8,2) .eq. 0) then !right hand rule, need to add 180 so Z positive is up
            if (head1(12,4) .eq. 0.0) then ! down
              int_head(13) = 180
            else  
              int_head(13) = 0
            end if
          else
            int_head(13) = int(head1(12,4))   ! no need to switch polarity
          end if
        else ! a horizontal component
          int_head(14) = int(head1(12,3))        
        end if
            
        int_head(17) = n_ts
      
        int_sac_null = -12345
        do i = 1, 48
          if (int_head(i) .eq. int_sac_null) then
            int_head(i) = int_null  ! convert from sac null to smc null values
          end if
        end do
        
        real_null = 1.7e+38
        do i = 1, 50
          real_head(i) = real_null
        end do

!        real_sac_null = -12345.0
!        do i = 1, 14
!          do j = 1, 5
!            if(head1(i,j) .eq. real_sac_null) then
!              head1(i,j) = real_null
!            end if
!          end do
!        end do
        
        real_head(2) = sps
        real_head(3) = head1(8,1) !event lat
        real_head(4) = head1(8,2) !event long
        real_head(5) = head1(8,4) !event depth
        real_head(9) = head1(8,5) !event mag (unknown type)
        real_head(11) = head1(7,2) !sta lat
        real_head(12) = head1(7,3) !sta long
        
        real_sac_null = -12345.0
        do i = 1, 50
          if (real_head(i) .eq. real_sac_null) then
            real_head(i) = real_null  ! convert from sac null to smc null values
          end if
        end do
        
      
      comments = ' '
        
      ncomments = 0

      ncomments = ncomments + 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      comments(ncomments) = '| File '//f_sac(1:nc_f_sac)//
     : ' reformatted by program SAC2SMC'
      ncomments = ncomments + 1
      comments(ncomments) = '| Data values scaled by the factor '
      call trim_c(comments(ncomments), nc_c)
      write(comments(ncomments)(nc_c+1:nc_c+11),'(0p,1x,e10.3)')
     :  xfactr
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      
      ncomments = ncomments + 1
      comments(ncomments) = '| Real headers in SAC file: '
      do i = 1, 14
        ncomments = ncomments + 1
        comments(ncomments) = ' '
        write(comments(ncomments),'(a1,5g15.7)')
     :                      '|',(head1(i,j),j=1,5)
      end do
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      
      ncomments = ncomments + 1
      comments(ncomments) = '| Integer headers in SAC file: '
      do i = 1, 8
        ncomments = ncomments + 1
        comments(ncomments) = ' '
        write(comments(ncomments),'(a1,5(2x,i9))')
     :                      '|',(head2(i,j),j=1,5)
      end do
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      

      ncomments = ncomments + 1
      comments(ncomments) = '| Character headers in SAC file: '
      do i = 1, 8
        ncomments = ncomments + 1
        comments(ncomments) = '|'//
     :                        head3(i,1)//head3(i,2)//head3(i,3)
      end do       
      ncomments = ncomments + 1
      comments(ncomments) = '|'
       
      int_head(16) = ncomments

      f_smc = ' '
      f_smc = f_sac(1:nc_f_sac)//'.smc'
      call trim_c(f_smc,nc_f_smc)

      write(*,'(a)') ' Writing file '//f_smc(1:nc_f_smc)

      precision = ' '
      precision = 'HIGH'
      int_head(47) = int_head(1)

      if (head2(4,2) .eq. 2) then
         itype = 4
      else if (head2(4,2) .eq. 3) then
         itype = 3
      else if (head2(4,2) .eq. 5) then
         itype = 1
      else
         itype = 0
      end if

      call smcwrite(f_smc,       precision,
     :                 ts, char_head, itype, 
     :              int_head, real_head, comments)
     
      DEALLOCATE(ts)




      close(nu_sac)

      go to 1234

4321  continue

      close(nu_ctl)
      close(nu_sum)

      stop
      end

      include '\forprogs\skip.for'
      include '\forprogs\distaz.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\skipcmnt.for'
      
      include '\forprogs\byte_swaps.F90'
      include '\forprogs\putspace.for'
 
