
       Program Taiwan_one_component2SMC

*  Reads in ASCII files from Willie Lee from the
*  26 Dec 06 Taiwan quake and reformats them
*  into SMC format 

*!Control file for program Taiwan_one_component2SMC
*!Name of summary file:
* Taiwan_one_component2SMC.sum
*!Quake Params: date, time (iyr, imon, idom, ihr, imin)
* 2006 12 26 12 26
*!Quake Params: location, magnitude (lat, long, h, Mw)
* 21.69  120.56  44  6.96 
*!Input file name
*P1_K042E.ASC
*P1_K042N.ASC
*P1_K042V.ASC
*stop
 
*  Dates:  11/19/07 - Written by D. Boore, based on CWB2SMC
*          03/29/08 - Expand station names
*          05/27/08 - Corrected expansion of "T" to "TCU"
*                     rather than "TTN".
 
      character precision*20
      character cmnts2skip(200)*79, read_fmt*50, 
     :          buf10*10, buf*80, buf4*4
       
      
      real  rhead(50), sps, orient_h, orient_v, eqmag, 
     :      eqlat, eqlong, rnull

      integer ihead(48), iyr, imon, idom, idoy, ihr, isec,
     :        ncomments, inull

      integer idata(100000) 
      real rdata(100000)

      character f_ctl*100, f_sum*100, f_smc*100, f_in*100 

      character chead(11)*80, comp_c*1, staname*10, 
     :          comments(50)*80, ins_type*5, 
     :          type_c*40, temp_c*10

      logical file_exist

      inull = -32768
      rnull = +1.7e+38

* Open control file:

      file_exist = .false.
      do while (.not. file_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file'//
     :    ' (cr = taiwan_one_component2smc.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'taiwan_one_component2smc.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=file_exist)
        if (.not. file_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      
      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum, file=f_sum(1:nc_f_sum), status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      read(nu_ctl,*) iyr, imon, idom, ihr, imin
      write(nu_sum,*) iyr, imon, idom, ihr, imin
      call DOY(iyr, imon, idom, idoy, istat)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      read(nu_ctl,*) eqlat, eqlong, eqdepth, eqmag
      write(nu_sum,*) eqlat, eqlong, eqdepth, eqmag

       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  

1000  continue

        buf = ' '
        read(nu_ctl,'(a)') buf
        call trim_c(buf,nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call trim_c(buf4,nc_buf4)
        call upstr(buf4)
      
        if (buf4(1:4) .eq. 'STOP') then
          close(nu_ctl)
          close(nu_sum)
          stop
        end if
      
        f_in = ' '
        f_in = buf(1:nc_buf)
        call trim_c(f_in, nc_f_in)
        call get_lun(nu_in)        
        open(unit=nu_in, file=f_in(1:nc_f_in), status='unknown')
        
        call skip(nu_in, 2)
        
        ins_type = ' '
        read(nu_in,'(a)') ins_type
        call trim_c(ins_type, nc_ins_type)
        
        staname = ' '
        read(nu_in,'(a)') staname
        call trim_c(staname, nc_staname)
        call upstr(staname)
        temp_c = ' '
        temp_c = staname(1:nc_staname)
        
        if (nc_staname .eq. 4) then ! an abbreviated name
          if (staname(1:1) .eq. 'B') then
             staname = 'TRB'//temp_c(2:4)
          else if (staname(1:1) .eq. 'C') then
             staname = 'CHY'//temp_c(2:4)
          else if (staname(1:1) .eq. 'D') then
             staname = 'MND'//temp_c(2:4)
          else if (staname(1:1) .eq. 'H') then
             staname = 'HWA'//temp_c(2:4)
          else if (staname(1:1) .eq. 'I') then
             staname = 'ILA'//temp_c(2:4)
          else if (staname(1:1) .eq. 'K') then
             staname = 'KAU'//temp_c(2:4)
          else if (staname(1:1) .eq. 'M') then
             staname = 'MTN'//temp_c(2:4)
          else if (staname(1:1) .eq. 'N') then
             staname = 'TTN'//temp_c(2:4)
          else if (staname(1:1) .eq. 'P') then
             staname = 'TAP'//temp_c(2:4)
          else if (staname(1:1) .eq. 'T') then
             staname = 'TCU'//temp_c(2:4)
          else
             print *,' No correspondence found with '//
     :               'station name '//staname(1:nc_staname)
             write(nu_sum,'(a)') ' No correspondence found with '//
     :               'station name '//staname(1:nc_staname)
          end if
          call trim_c(staname, nc_staname)
        end if
     

        comp_c = ' '
        read(nu_in,'(a)') comp_c
        call upstr(comp_c)
        call trim_c(comp_c, nc_comp_c)
        
        read(nu_in,*) stalat
        
        read(nu_in,*) stalong
        
        read(nu_in,*) staelev
        
        read(nu_in,'(i4,1x,i2,1x,i2,2x,i2,1x,i2,1x,i2)')
     :              iyr, imon, idom, ihr, imin, isec
        
        read(nu_in,*) npts
        
        read(nu_in,*) sps
        
        read(nu_in,*) counts_per_g
        
        buf10 = ' '
        read(nu_in,'(a)') buf10
        call trim_c(buf10,nc_buf10)
        read_fmt = ' '
        read_fmt = '('//buf10(1:nc_buf10)//')'
        call trim_c(read_fmt, nc_read_fmt)
        
        call skip(nu_in,1)
        
        read(nu_in,read_fmt(1:nc_read_fmt)) (idata(i), i = 1, npts)
        
        close(nu_in)
        
        factor = 981.0/counts_per_g
        
        do i = 1, npts
          rdata(i) = factor*float(idata(i))
        end do
        
* Get peak motions:
      
        call mnmaxidx(rdata, 1, npts, 1,
     :             ymin, ymax, indx_min, indx_max) 
        pga = amax1(abs(ymin), abs(ymax))
        tmin = float(indx_min-1)/sps
        tmax = float(indx_max-1)/sps
        rhead(29) = tmax
        rhead(30) = ymax
        rhead(31) = tmin
        rhead(32) = ymin

       call distaz( -1.0, eqlat, eqlong, stalat, stalong,
     : rdeg, rkm, az, baz)
 
 
        if(comp_c(1:1) .eq. 'V') then
          orient_v = 0.0
          orient_h = 0.0
          indx_comp = 2
        else if(comp_c(1:1) .eq. 'N') then
          orient_v = 90.0
          orient_h = 0.0
          indx_comp = 1
        else if(comp_c(1:1) .eq. 'E') then
          orient_v = 90.0
          orient_h = 90.0
          indx_comp = 3
        else
          write(nu_sum,'(3a)') 
     :      ' comp_c = ',
     :      ' comp_c(1:1) = ',
     :      '; .ne. V, N, or E: QUITTING!!!'
          stop
*          orient_v = rnull
*          orient_h = rnull
        end if

 
* Write headers:

        chead(1) = '*'
        chead(2) = '*'

        write(chead(3),'(a)') 
     :    staname(1:nc_staname)

        write(chead(4),'(5x,i4,2x,i2,2x,i2,4x,i2,i2,1x,a)') 
     :              iyr, imon, idom, ihr, imin,'Taiwan'
        chead(5) = 
     :       'Moment Mag=          Ms=          Ml='
        write(chead(5)(12:20),'(f9.2)') eqmag
        write(chead(5),'(a,6x,f3.1)') 'Moment Mag=', eqmag
        chead(6) = 
     :    'station =                                component='
        chead(6)(11:nc_staname+10) = 
     :     staname(1:nc_staname)
        if (orient_v .eq. 0.0) then      ! assume vertical
          chead(6)(56:57) = 'UP'
        else
          write(chead(6)(53:57),'(i5)') int(orient_h)
        end if
        write(chead(7),'(a,t34,a)') 'epicentral dist =','pk acc ='
        write(chead(7)(18:26),'(f9.1)') rkm
        write(chead(8),'(a, t22, a)') 
     :      'inst type='//ins_type(1:nc_ins_type), 'data source = CWB'
        chead(9) = '*'
        chead(10) = '*'
        chead(11) = '*'

* Set up comments:
        comments(1) = '| CWB file '//f_in
        comments(2) = '| Reformatted to SMC format, '//
     :                'using Taiwan_one_component2smc.for'
        ncomments = 2
             
* Now fill the integer header block:
* First fill with null values:
        do i = 1, 48
          ihead(i) = inull
        end do

        ihead(2) = iyr
        ihead(3) = idoy
        ihead(4) = ihr
        ihead(5) = imin
        ihead(6) = isec
        ihead(9) = indx_comp
        ihead(13) = orient_v
        ihead(14) = orient_h
        ihead(16) = ncomments
        ihead(17) = npts
 
      precision = ' '
      precision = 'HIGH'

* Now fill the real header block:
* First fill with null values:
        do i = 1, 50
          rhead(i) = rnull
        end do
   
        rhead(2) = sps
        rhead(3) = eqlat
        rhead(4) = eqlong
        rhead(5) = eqdepth
        rhead(6) = eqmag
        rhead(11) = stalat
        rhead(12) = stalong
        rhead(13) = staelev
        rhead(17) = rkm
        rhead(18) = az
  
         
 
      if (precision(1:4) .eq. 'HIGH') then
        f_smc = f_in(1:nc_f_in)//'.smc8'
        ihead(47) = 8
      else
        f_smc = f_in(1:nc_f_in)//'.smc'
        ihead(47) = ihead(1)
      end if
      
      call trim_c(f_smc,nc_f_smc)

      write(*,'(a)') ' Writing file '//f_smc(1:nc_f_smc)

      itype = 1
      call smcwrite(f_smc, precision,
     :              rdata, chead, itype, 
     :              ihead, rhead, comments)
     
* That should be it; loop back for another component

      go to 1000

 
      end

      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\doy.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\distaz.for'
      include '\forprogs\csrc.for'
      include '\forprogs\csrf.for'
      include '\forprogs\csri.for'
      include '\forprogs\csr123.for'



