      Program Wrapped2SMC

* Reformat into a smc file an ascii file in which the data are
* in a block with a known format (e.g., (10f8.2)). 
* The sample rate of the data is specified in the
* control file.
 
* Sample control file:
*!Control file for Wrapped2SMC      ! first line
*!Revision of program involving a change in the control file on this date:
*   06/27/10
*!Name of summary file:
* wrapped2smc.sum
*!Headers to skip before data:
* 16
*!Xfactr (multiply input acceleration values by this factor; 
*!  e.g., specify 981.0 to convert from g to cm/s/s):
* 1.0
*!sps (0 => obtain from one of the headers):
*  0.0
*!Header row and column containing epicenter latitude (row=0 => not available):
*  4 11
*!Format of epicenter latitude (need a dummy entry even if not used):
* (f7.3)
*!Header row and column containing epicenter longitude (row=0 => not available):
*  4 19
*!Format of epicenter longitude (need a dummy entry even if not used):
* (f7.3)
*!Header row and column containing magnitude (row=0 => not available):
*  5 12
*!Format of magnitude (need a dummy entry even if not used):
* f3.1
*!Header row and column containing station code (row=0 => not available):
*  6 18
*!Format of station code (need a dummy entry even if not used):
* (a6)
*!Header row and column containing station latitude (row=0 => not available):
*  0 11
*!Format of station latitude (need a dummy entry even if not used):
* (f7.3)
*!Header row and column containing station longitude (row=0 => not available):
*  0 19
*!Format of station longitude (need a dummy entry even if not used):
* (f7.3)
*!Header row and column containing orientation (assume "EW", "NS", or "UD") (row=0 => not available):
*  10 7
*!Header row and column containing low-pass filter corner frequency (row=0 => not available):
*  0 19
*!Format of low-pass filter corner frequency (need a dummy entry even if not used):
* (f7.3)
*!Header row and column containing number of points (row=0 => not available)
*  12 16
*!Format of npts (need a dummy entry even if not used):
*  i7
*!Header row and column containing sample spacing (row=0 => not available)
*  12 55
*!Format of sample spacing dt (need a dummy entry even if not used):
*  (f5.3)
*!Number of data points per line (assumes that the entries are data
*!  only, not time, data pairs):
* 8
*!Use old (standard) smc format or new
*!  higher precision format.   Specify "high" for
*!  high precision; any other word defaults to standard
*!  precision (but some word is needed as a placholder, even if
*!  standard precision is desired):
* high
*!Input file name (time,data pairs; "stop" in any column to quit):
*chnua370001.dat
*chnua370002.dat
*chnua370003.dat
*chnua370004.dat
*chnua370005.dat
*chnua370006.dat
*chnua370007.dat
*chnua370008.dat
*chnua370009.dat
*STOP

 
* Dates: 01/18/08 - Written by D. Boore, patterned after asc2smc
!        05/21/10 - Read only number of data points per line, and compute
!                   npts
!        06/27/10 - Revise to allow more general extraction of npts, dt, etc from headers
!                   (this revision was motivated by my need to reformat data from
!                   the Wenchuan earthquake).
!        11/11/20 - Allow longer input and output file names (previously 40 characters)
 
      parameter (ndimen = 200000, max_npts_per_line = 100)
      real time(ndimen), ts(ndimen)
      character f_ctl*60, f_sum*60, f_asc*100, f_smc*100, f_asc_4*4
      character char_head(11)*80, comments4smc(100)*80, headers(100)*79
      character precision*20, cmnts2skip(50)*80
      character buf*120
      character date_ctl_correct*8, date_ctl_in*30
      integer int_head(48)
      real real_head(50), dum(max_npts_per_line)
      
      integer 
     :   row_epilat, col_epilat, 
     :   row_epilong, col_epilong, 
     :   row_stalat, col_stalat, 
     :   row_stalong, col_stalong, 
     :   row_mag, col_mag, 
     :   row_stacode, col_stacode, 
     :   row_orient, col_orient, 
     :   row_flc, col_flc, 
     :   row_npts, col_npts, 
     :   row_dt, col_dt

      real epilat, epilong, stalat, 
     :   stalong, mag, flc, dt
     
      integer npts 

      character stacode*6, orient*4
      character epilat_format*10, epilong_format*10
      character stalat_format*10, stalong_format*10
      character mag_format*10 
      character stacode_format*10 
      character npts_format*10 
      character flc_format*10 
      character dt_format*10 
      
      logical f_exist

      do i = 2, 11
        char_head(i) = '*'
      end do

      ndimen_max = 200000
      
      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file ("Enter" = wrapped2smc.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'wrapped2smc.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '06/27/10'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum, file=f_sum(1:nc_f_sum), status='unknown')
      write(nu_sum,'(a)') '  Summary program for program WRAPPED2SMC'      

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl, *) nheaders2read
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) nheaders2read

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) xfactr
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) xfactr

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) sps
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) sps

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_epilat, col_epilat
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_epilat, col_epilat
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      epilat_format = ' '
      read(nu_ctl,'(a)') epilat_format
      call trim_c(epilat_format, nc_epilat_format)
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, '(a)') epilat_format(1:nc_epilat_format)
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_epilong, col_epilong
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_epilong, col_epilong
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      epilong_format = ' '
      read(nu_ctl,'(a)') epilong_format
      call trim_c(epilong_format, nc_epilong_format)
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, '(a)') epilong_format(1:nc_epilong_format)
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_mag, col_mag
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_mag, col_mag
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      mag_format = ' '
      read(nu_ctl,'(a)') mag_format
      call trim_c(mag_format, nc_mag_format)
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, '(a)') mag_format(1:nc_mag_format)
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_stacode, col_stacode
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_stacode, col_stacode
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      stacode_format = ' '
      read(nu_ctl,'(a)') stacode_format
      call trim_c(stacode_format, nc_stacode_format)
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, '(a)') stacode_format(1:nc_stacode_format)
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_stalat, col_stalat
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_stalat, col_stalat
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      stalat_format = ' '
      read(nu_ctl,'(a)') stalat_format
      call trim_c(stalat_format, nc_stalat_format)
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, '(a)') stalat_format(1:nc_stalat_format)
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_stalong, col_stalong
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_stalong, col_stalong
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      stalong_format = ' '
      read(nu_ctl,'(a)') stalong_format
      call trim_c(stalong_format, nc_stalong_format)
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, '(a)') stalong_format(1:nc_stalong_format)
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_orient, col_orient
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_orient, col_orient
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_flc, col_flc
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_flc, col_flc
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      flc_format = ' '
      read(nu_ctl,'(a)') flc_format
      call trim_c(flc_format, nc_flc_format)
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, '(a)') flc_format(1:nc_flc_format)
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_npts, col_npts
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_npts, col_npts
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      npts_format = ' '
      read(nu_ctl,'(a)') npts_format
      call trim_c(npts_format, nc_npts_format)
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, '(a)') npts_format(1:nc_npts_format)
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) row_dt, col_dt
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) row_dt, col_dt
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      dt_format = ' '
      read(nu_ctl,'(a)') dt_format
      call trim_c(dt_format, nc_dt_format)
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, '(a)') dt_format(1:nc_dt_format)
             
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) npts_per_line
      write(nu_sum,'(a)') cmnts2skip(1:nc_cmnts2skip)     
      write(nu_sum, *) npts_per_line
      
      if(npts_per_line > max_npts_per_line) then
        write(*,*) ' ERROR: npts_per_line > max_npts_per_line (',
     :             max_npts_per_line,'); QUITTING!!!'
        STOP
      end if

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      precision = ' '
      read(nu_ctl,'(a)') precision
      call trim_c(precision, nc_precision)
      call upstr(precision(1:nc_precision))

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

! Read the next file:

      loop over files: DO

        f_asc = ' '
        read(nu_ctl,'(a)') f_asc
        call trim_c(f_asc,nc_f_asc)
        f_asc_4 = ' '
        f_asc_4 = f_asc(1:4)
        call upstr(f_asc_4)
        if(f_asc_4 == 'STOP') EXIT
        
        f_exist = .false.
        inquire(file=f_asc(1:nc_f_asc), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE '//f_asc(1:nc_f_asc)//
     :        ' DOES NOT EXIST *******, going to next file' 
          CYCLE
        end if
  
  
        write(nu_sum,'(2a)') '  Processing file: ', f_asc(1:nc_f_asc)

        print *,' '
        print *,' Processing file: '//f_asc(1:nc_f_asc)
  
        call get_lun(nu_asc)
        open(unit=nu_asc, file=f_asc(1:nc_f_asc), status='unknown')

        comments4smc = ' '
        headers = ' '
        
        ncomments = 0
        ncomments = ncomments + 1
        comments4smc(ncomments) = '|'
        ncomments = ncomments + 1
        comments4smc(ncomments) = '| File '//f_asc(1:nc_f_asc)//
     : ' reformatted by program Wrapped2SMC'
        ncomments = ncomments + 1
        comments4smc(ncomments) = '|'

        
        if (nheaders2read /= 0) then
          do i = 1, nheaders2read
            read(nu_asc,'(a)') headers(i)
            ncomments = ncomments + 1
            comments4smc(ncomments) = '|'//headers(i)
          end do
          ncomments = ncomments + 1
          comments4smc(ncomments) = '|'
        end if
          
        int_null = -32768
        int_head  = int_null
  
        real_null = 1.7e+38
        real_head  = real_null
 
! Extract metadata:

        if (row_epilat /= 0) then
          call trim_c(headers(row_epilat), nc_headers)
          read(headers(row_epilat)(col_epilat: nc_headers), 
     :              epilat_format(1:nc_epilat_format)) epilat
          read(headers(row_epilong)(col_epilong: nc_headers), 
     :              epilong_format(1:nc_epilong_format)) epilong
          real_head(3) = epilat
          real_head(4) = epilong
        end if
             
        if (row_mag /= 0) then
          call trim_c(headers(row_mag), nc_headers)
          read(headers(row_mag)(col_mag: nc_headers), 
     :              mag_format(1:nc_mag_format)) mag
          real_head(6) = mag
        end if
             
        if (row_stacode /= 0) then
          stacode = ' '
          call trim_c(headers(row_stacode), nc_headers)
          read(headers(row_stacode)(col_stacode: nc_headers), 
     :              stacode_format(1:nc_stacode_format)) stacode
          call trim_c(stacode, nc_stacode)
        end if
        
        if (row_stalat /= 0) then
          call trim_c(headers(row_stalat), nc_headers)
          read(headers(row_stalat)(col_stalat: nc_headers), 
     :              stalat_format(1:nc_stalat_format)) stalat
          read(headers(row_stalong)(col_stalong: nc_headers), 
     :              stalong_format(1:nc_stalong_format)) stalong
          real_head(11) = stalat
          real_head(12) = stalong
        end if
             
        if (row_orient /= 0) then
          orient = ' '
          call trim_c(headers(row_orient), nc_headers)
          read(headers(row_orient)(col_orient: nc_headers), 
     :              '(a2)') orient
          if (orient(1:2) == 'EW') then
            int_head(14) = 90.0
          else if (orient(1:2) == 'NS') then
            int_head(14) = 0.0
          else if (orient(1:2) == 'UD') then
            int_head(13) = 0.0
          else
            write(nu_sum,'(a)') ' orient = '//orient(1:2)//
     :          ' not a recognized value (EW, NS, or UD)'
          end if
          call trim_c(orient, nc_orient)
        end if
        
        if (row_flc /= 0) then
          call trim_c(headers(row_flc), nc_headers)
          read(headers(row_flc)(col_flc: nc_headers), 
     :              flc_format(1:nc_flc_format)) flc
          real_head(33) = flc
        end if
             
        if (row_npts /= 0) then
          call trim_c(headers(row_npts), nc_headers)
          read(headers(row_npts)(col_npts: nc_headers), 
     :              npts_format(1:nc_npts_format)) npts
          int_head(17) = npts
        end if
             
        if (row_dt /= 0) then
          call trim_c(headers(row_dt), nc_headers)
          read(headers(row_dt)(col_dt: nc_headers), 
     :              dt_format(1:nc_dt_format)) dt
          write(nu_sum,'(a, 1x,f7.4, 1x,f5.1)') ' dt, sps = ', dt, sps
          sps = 1.0/dt
          real_head(2) = sps
        end if
             
 
        n_ts = 0
        read_another_block_of_data: DO
          buf = ' '
          READ(nu_asc,'(a)',iostat=iread_status) buf
          call trim_c(buf, nc_buf)
          IF (iread_status < 0) THEN  ! no more lines of data
            EXIT read_another_block_of_data
          END IF
!DEBUG
!          write(*,*) ' buf: '//buf(1:nc_buf)
!DEBUG

          READ(buf(1:nc_buf),*, iostat=iread_status) 
     :               (ts(n_ts+i), i = 1, npts_per_line)
          IF (iread_status < 0) THEN  ! fewer data points than npts_per_line
          
            write(*,*) ' Dealing with last line:'
! deal with last line
            iend = npts_per_line - 1
            DO
              write(*,*) ' iend = ', iend
              READ(buf(1:nc_buf),*, iostat=iread_status_last_line) 
     :               (dum(i), i = 1, iend)
              IF (iread_status_last_line < 0) THEN  ! fewer data points than iend, reduce iend and try again
                iend = iend - 1
                CYCLE
              END IF
              DO i = 1, iend
                n_ts = n_ts + 1
                ts(n_ts) = dum(i)
              END DO
              EXIT
            END DO
            EXIT read_another_block_of_data
          END IF
          n_ts = n_ts + npts_per_line
          IF (n_ts == ndimen_max) THEN
            EXIT read_another_block_of_data
          END IF
        END DO read_another_block_of_data
        
!*DEBUG 
        write(*,*) ' n_ts, ts(1), ts(n_ts) = ',
     :               n_ts, ts(1), ts(n_ts)
!*DEBUG  
        
! Scale the time series:
        do i = 1, n_ts
          ts(i) = xfactr * ts(i)
        end do
  
        if (row_dt /= 0) then
          if (npts /= n_ts) then
            write(nu_sum,*)
            write(nu_sum,'(a)') ' ***WARNING***, npts /= n_ts ***'
            write(nu_sum,'(a,1x,i6,1x,i6,1x,f7.2)') 
     :          ' npts, n_ts, sps = ', npts, n_ts, sps
            write(nu_sum,*)
          end if
        end if 
        write(nu_sum,'(a,1x,i6,1x,f7.2)') '    n_ts, sps = ', n_ts, sps
  
        int_head(17) = n_ts
        
        real_head(2) = sps

        ncomments = ncomments + 1
        comments4smc(ncomments) = '| Data values scaled by the factor '
        call trim_c(comments4smc(ncomments), nc_c)
        write(comments4smc(ncomments)(nc_c+1:nc_c+11),'(0p,1x,e10.3)')
     :  xfactr
        ncomments = ncomments + 1
        comments4smc(ncomments) = '|'

        int_head(16) = ncomments

        f_smc = ' '

        f_smc = f_asc(1:nc_f_asc)
        nc_f_smc = nc_f_asc
        
        if (row_stacode /= 0) then
          f_smc = f_smc(1:nc_f_smc)//'.'//stacode(1:nc_stacode)
        end if
        call trim_c(f_smc, nc_f_smc)
        
        if (row_orient /= 0) then
          f_smc = f_smc(1:nc_f_smc)//'.'//orient(1:nc_orient)
        end if
        call trim_c(f_smc, nc_f_smc)
       

        if (precision(1:4) .eq. 'HIGH') then
          f_smc = f_smc(1:nc_f_smc)//'.smc8'
        else
          f_smc = f_smc(1:nc_f_smc)//'.smc'
        end if
        
        call trim_c(f_smc,nc_f_smc)
  
        write(*,'(a)') ' Writing file '//f_smc(1:nc_f_smc)

        call smcwrite(f_smc, precision,
     :              ts, char_head, 0, 
     :              int_head, real_head, comments4smc)
     
        close(nu_asc)

      END DO loop over files
  
 
      close(nu_ctl)
      close(nu_sum)

      stop
      end

      include '\forprogs\skip.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\distaz.for'
